//
//TXOUT drives serial data out to SOUT
//
`timescale 1 ps / 1 ps
module altr_uart_txout #(
    parameter DSIZE = 8
) (/*port details*/
   //Clock and Reset
   input  clk,
   input  rst_n,
   input  baud_clken,

   //CSR Inputs
   input  lcr_break,
   input  lcr_pen_l,
   input  lcr_sp_l,
   input  lcr_eps_l,
   input  lcr_stop_l,
   input  [2:0] lcr_dls_l,

   //Consolidated avail input from TXSTOR or TXFC
   input  txout_avail,

   //Consolidated datapath from TXSTOR and TXFC
   input  [DSIZE-1:0] tx_data,

   //Get output from TXOUT
   output txout_get,

   //RXFC FSM - XON state
   input  rxfc_xon,

   //TXFSM Idle state
   output txfsm_idle,

   //SOUT signal
   output sout_pre,
   output sout_oe_pre
);

//txfsm internal signals
logic txfsm_start;
logic txfsm_data;
logic txfsm_parity;
logic txfsm_stop;

//bit_timer internal signals
logic [3:0] bit_timer;
logic bit_timer_inc;
//logic bit_timer_zero;
logic one_bit_time;
logic one_bit_time_m1;
logic half_bit_time;
logic half_bit_time_m1;

//data_cnt internal signals
logic data_cnt_s_rst;
logic data_cnt_inc;
logic data_cnt_full;

//get internal signal
logic load_r;

//tx_shift_reg controls
logic load;
logic shift;

//------------------------------------------------------------------------------
//TXFSM Logic
//------------------------------------------------------------------------------
altr_uart_txfsm txfsm (.*);

//------------------------------------------------------------------------------
//TX_BIT_TIMER Logic
//------------------------------------------------------------------------------

//Tx Bit Timer Increment Term
assign bit_timer_inc = txfsm_start | txfsm_data | txfsm_parity | txfsm_stop;

//Tx Bit Timer Outputs
//assign bit_timer_zero   = (bit_timer == 4'h0);
assign one_bit_time     = (bit_timer[3] == 1 && bit_timer[2] == 1 && bit_timer[1] == 1 && bit_timer[0] == 1) ? 1'b1 : 1'b0;
assign one_bit_time_m1  = (bit_timer[3] == 1 && bit_timer[2] == 1 && bit_timer[1] == 1 && bit_timer[0] == 0) ? 1'b1 : 1'b0;
assign half_bit_time    = (bit_timer[3] == 1 && bit_timer[2] == 0 && bit_timer[1] == 0 && bit_timer[0] == 0) ? 1'b1 : 1'b0;
assign half_bit_time_m1 = (bit_timer[3] == 0 && bit_timer[2] == 1 && bit_timer[1] == 1 && bit_timer[0] == 1) ? 1'b1 : 1'b0;


//Tx Bit Timer Instantiation
altr_uart_bit_timer tx_bit_timer (
   .inc(bit_timer_inc),
   .s_rst(~bit_timer_inc),
   .*
);
  
//------------------------------------------------------------------------------
//TX_DATA_COUNTER Logic
//------------------------------------------------------------------------------

assign data_cnt_s_rst = ~txfsm_data | data_cnt_full;
assign data_cnt_inc = txfsm_data & one_bit_time;

//Tx Data Counter Instantiation
altr_uart_data_cnt tx_data_cnt (
   .s_rst(data_cnt_s_rst),
   .inc(data_cnt_inc),
   .data_cnt(),
   .*
);

//------------------------------------------------------------------------------
//TXOUT Get Logic
//------------------------------------------------------------------------------

//txout_get drains 1 character from TXSTOR
// txout_get runs in "clk" domain while 
// load runs in "baud_clken" domain
// txout_get should only assert after load
// use falling edge detect on load for txout_get
always_ff @(posedge clk or negedge rst_n) begin
   if (!rst_n) load_r <= 0;
   else        load_r <= load;
end

//txout_get has additional qualifiers to take into account possibility that txout_avail
//can deassert before txout_get
//txout_get uses a combination of logic in both baud_clk and clk domain
assign txout_get = ~load & load_r & txfsm_start & txout_avail;

//------------------------------------------------------------------------------
//TX_SHIFT_REG Logic
//------------------------------------------------------------------------------

//load signals is now exactly the same as the signal that triggers
//the transition from IDLE to START state for TXOUT FSM
//load = arc_IDLE_START
//load uses a combination of logic in both baud_clk and clk domain
assign load = txfsm_idle & txout_avail & rxfc_xon;

//shift logic
assign shift = txfsm_data & one_bit_time;

//tx_shift_reg instantiation
altr_uart_tx_shift #(
   .DSIZE(DSIZE)
) tx_shift (.*);

//------------------------------------------------------------------------------
//SOUT_EN logic
//------------------------------------------------------------------------------

//Beta-feature (Fogbugz: 112424)
//Signal is used to turn on transmit buffer
assign sout_oe_pre = ~txfsm_idle;

endmodule: altr_uart_txout
